"""Kukur source for PI Data Archives using PI Web API."""

# SPDX-FileCopyrightText: 2022 Timeseer.AI
# SPDX-License-Identifier: Apache-2.0

from .piwebapi_da import from_config

__all__ = ["from_config"]
